#! /usr/bin/python
################################################################################
#
# File:         nph-bibsearchcgi0.py
# RCS:          $Header: /u/testbed/CVSROOT/dldev/src/interbib2007/Scripts/nph-bibsearchcgi0.py,v 1.7 2007/07/31 07:16:36 paepcke Exp $
# Description:  Bibsearch CGI Front-end
# Author:       Arturo Crespo, Stanford University
# Created:      Thu Jul  4 12:42:34 1996
# Modified:     Tue Jul 31 00:15:57 2007 (Andreas Paepcke) paepcke@SNUGGLES
# Language:     Python
# Package:      N/A
# Status:       Experimental (Do Not Distribute)
#
# (c) Copyright 1996, Stanford University, all rights reserved.
#
################################################################################
#
# Revisions:
#
# Thu Jul  4 12:43:22 1996 (Andreas Paepcke) paepcke@Phoebe.Stanford.EDU
#  Replaced use of module posix by os for portability
################################################################################
#
# Bibsearch CGI Front-end
#
# Arturo Crespo  (crespo@cs.stanford.edu)
#
# Description:
#   This module provides a simple CGI-BIN front end to bibsearch.
#   It produces a HTML form suitable for proccessing by nph-bibsearch.py
#   It can optionally take the parameter "directory" which becomes the
#   default value for the directory containing the bibliography.
#

import sys

# prepare to respond the request
# we do this here to be able to echo back errors when importing
# all needed modules
sys.stdout.write('HTTP/1.0 200 0K\n')
sys.stdout.write('Server: NCSA/1.4.1\n')
sys.stdout.write('Content-type: text/html\n\n')
# echo all errors to the user
sys.stderr = sys.stdout

import bibsearch
import cgi
import bibrefgen
import bibcompile
import os
import string

#
# printHeader
#   IN:
#   OUT:
#   DESC: prints a header for the bibsearch form
#

def printHeader():
  print """
  <table width=100% border=0>
  <tr>
    <td>
      <img src="../gifs/interbibSearch.gif"
                      align=absmiddle
                      height=140 width=100
                      hspace=20
                      border=0/></td>
    <td>
      <h1> 
        InterBib: Search Engine
      </h1></td>
    <td>
      <img src="../gifs/dllogo-smaller-xparent.gif" align=absmiddle/>
    </td>
    <td>
       <a href="../bibsearchHelp.html">
         <img src="../gifs/interbibHelp.gif"
              align=baseline
              height=40 width=40
              border=0/></td>
  </tr>
  </table>
  <hr/>"""

#
# printHeBibSearchForm
#   IN:  directory  -  optional cgi variable for the directory containing
#                      interbib bibliographies.
#   OUT:
#   DESC: prints a header for the bibsearch form
#         Assumes that the reply header was printed before calling it
#         

def printBibSearchForm():

  # print a nice logo
  printHeader()

  # get the cgi variable directory, if not existent assign the default
  try:
    cgiVars = cgi.parse()
    if 'directory' not in cgiVars.keys():
      cgiVars['directory'] = ["/u/testbed/public_html/interbib/bibliographies/"]
  except:
    cgiVars = {}
    cgiVars['directory'] = ["/u/testbed/public_html/interbib/bibliographies/"]

  # print a small form that allows to change the directory
  ###print '<form Method="GET" Action="/cgi-bin/nph-bibsearchcgi0.py">'
  ###print 'Directory: ' + '<input name="directory" size=40 type="text" value=' + cgiVars['directory'][0] + '>'
  ###print '<input type=Submit name="Directory" value="Change Directory">' + '<P>'
  ###print '</form>'

  # print the search form
  print '<form method="post" action="/cgi-bin/nph-bibsearchcgi.py">'
  try:	
    files = os.listdir(cgiVars['directory'][0])
  except:
    files = []
  files.sort()
  print '<center>'
  print '<strong>Available Bibliographies:</strong>'
  print '<table border=1>'
  print "<input type=hidden name=directory value=" + cgiVars['directory'][0] + ">"
  count = 0
  # Go through files in dir: find the ones w/o extension and assume they are InterBib
  # files. Only display any given file name X if (i) there is a corresponding index file
  # X.idx, and (ii) there is no lock file X.lck which would indicate that the indx file
  # is half done:
  for fileName in files:
    # If this is a file w/ extension, ignore it:
    if os.path.splitext(fileName)[1]:
      continue
    fullFileName = os.path.join(cgiVars['directory'][0], fileName)
    if os.path.isfile(fullFileName + '.idx') and not os.path.isfile(fullFileName + '.lck'):
      if (count%5==0):
        print "<tr>"
      try:
	# Get file length:
        fileLen = os.stat(fullFileName)[6]
        if fileLen > 1000000:
	  fileLen = str(fileLen/1000000) + 'MB'
        else:
	  fileLen = str(fileLen/1000) + 'KB'
      except:
	fileLen = ""
	
      print "<td>" + '<input type="checkbox" name=filename value=' + fileName + ' >' + fileName + ' <font size=-1>(' + fileLen + ')</font>' + "<br/></td>"
      if (count%5==4):
        print "</tr>"
      count = count + 1

  print '</table>'
  print '</center>'

  if (count == 0):
    print "No databases in this directory\n"

  print """
<center>
Databases available at: 'http://interbib.stanford.edu/~testbed/interbib/bibliographies/'
</center>
<hr/>

<table width=100%>
  <tr>
    <td align=left>
      <input type=Submit name='Alldb' value='Show all entries'/></td>
    <td align=right>
      <table border=1>
	<th colspan=2> Options</th>
	<tr>
	  <td> <input value="on" name="annote" type="checkbox"/>Include annotations </td>
	  <td> <input value="on" name="sortEntries" type="checkbox"/>Sort alphabetically </td></tr>
	<tr>
	  <td> <input value="on" name="exactMatch" type="checkbox"/>Exact match </td>
	  <td> <input value="on" name="caseSensitive" type="checkbox"/>Case sensitive </td></tr>
	<tr>
	  <td colspan=2> 
	    <table width=100% border=0>
	      <tr>
	        <td align=left> <input value="on" name="useIndex" type="checkbox" checked/>Use index</td>
	        <td align=right rowspan=2> (uncheck to search words with less than 4 chars or special symbols)</td></tr>
	      </tr><tr></tr>
	    </table></td></tr>
      </table>
  </tr>
  <tr>
    <td align=left>
      <input type=Submit name='Submit' value='Show only entries matching:'/></td>
  </tr>
</table>
  
<table>
  <tr>
    <td align=right> Any of entry keys: </td>
    <td> <input name="key" size=15 type="text"/><br/> </td>
  </tr>
  <tr>
    <td align=right> and any of title words: </td>
    <td> <input name="title" size=40 type="text"/> </td>
  </tr>
  <tr>
    <td align=right> and any of authors: </td>
    <td> <input name="author" size=40 type="text"> </td>
  </tr>
  <tr>
    <td align=right> and any of years: </td>
    <td> <input name="year" size=15 type="text"> </td>
  </tr>
  <tr>
    <td align=right> and any of field: </td>
    <td>
      <table>
        <tr>
          <td> <input name="otherField" size=15 type="text"> </td>
          <td> <input name="otherFieldValue" size=15 type="text"> </td>
        </tr>
      </table></td>
  </tr>

</table>

<p></p>

<P></p>

</form>

<a href="http://interbib/~testbed/interbib/interbib.html">
    <img src="../gifs/arrowLeft.gif" 
         align=absmiddle
         hspace=10
         border=0/><br/>
  InterBib Homepage

<p></p>
<!-- <a href="mailto:paepcke@cs.stanford.edu">Comments/bugs to: Andreas Paepcke</a> -->
</body>
</html>
"""

# 
#
# Main function
#
#

printHeader()
print "<center><i>Sorry, the search facility is broken. (Yes, I should hide the links that lead here. \
I didn't, though. I'm sure I will...Soon!)</i></center>"
#****printBibSearchForm();
sys.stdout.flush()

