#! /usr/bin/python
################################################################################
#
# File:         bibservicecgi.py
# RCS:          $Header: /u/testbed/CVSROOT/dldev/src/interbib2007/Scripts/nph-bibservicecgi.py,v 1.25 2007/08/07 21:38:40 paepcke Exp $
# Description:  cgi-bin script for bibliography services through the web
# Author:       Andreas Paepcke, Stanford University
# Created:      Wed Dec 20 16:18:29 1995
# Modified:     Tue Aug 07 14:36:51 2007 (Andreas Paepcke) paepcke@SNUGGLES
# Language:     Python
# Package:      N/A
# Status:       Experimental (Do Not Distribute)
#
# (c) Copyright 1995, Stanford University, all rights reserved.
#
################################################################################
#
# Revisions:
#
# Sat Jul 28 08:11:37 2007 (Andreas Paepcke) paepcke@EMIL
#  Disabled bib retention till it's fixed. See
#   #****** Needs debugging. self.handleBibFileRetention()in 
#   one of the run() methods.
# Thu Jul 26 08:24:55 2007 (Andreas Paepcke) paepcke@EMIL
#  Replaced 'verbos=1' with verbosity = 'quiet' in converter creation.
# Wed May  6 10:55:12 1998 (Andreas Paepcke) paepcke@Haddock.Stanford.EDU
#  When creating bib converter, added parm: verbose = 1 to get more feedback
# Thu Apr 30 09:40:26 1998 (Andreas Paepcke) paepcke@Haddock.Stanford.EDU
#  Made sure that DOS file names were treated correctly.
# Fri Jul 12 13:28:51 1996 (Andreas Paepcke) paepcke@Walrus
#  Changed settings of permissions from 'a+rw' to '644'
# Sun Jul  7 10:02:14 1996 (Andreas Paepcke) paepcke@Stingray
#  When moving interbib db into bibliography directory, added rw chmod
#  command.
# Sun Jul  7 10:01:37 1996 (Andreas Paepcke) paepcke@Stingray
#  Added automatic startup of indexing process
# Sun Jan 14 09:32:23 1996 (Andreas Paepcke) paepcke@AndreasHomePC
#  Added ability to set citation delimiters.
################################################################################

# Pretty horrible piece of code. Here's the scoop:
#
# - Both nph-bibconvertcgi.py and nph-bibdocxformcgi.py go through here
#   because they have a fair amount in common (their superclass)
# - This script is invoked three times in each case (convert or docxform):
#   (i) The form dict is created, with delay-feedback given to user because
#   this can involve large file xfers from client to server. At the end,
#   a Netscape FRAME HTML code is emitted which makes two windows and
#   requests an invocation of this script for each of the windows.
# - the two windows call the script with arg (ii) 'compute' and (iii)
#   'deliver', respectively. The first runs the code, the second synchs
#   with the compute invocation through a lock file, pumping info back as
#   fast as the compute invocation produces it.
# - The form dict is passed from invocation (i) to (ii) through a file which
#   then doubles as the output file that links invocation (ii) and (iii).
# - We use the multipart/x-mixed-replace to start a new page after the
#   step (i) above. We use non-parsed-header because we need to return
#   slow-moving feedback that would otherwise be buffered (I believe).
# - If you want to not automatically create bibliography index files
#   for incoming error-free bibliographies, set self.makeIndexes to 0 in
#   class bibService.


boundary = '|-------------------InterBib Service------|'
startBoundary = '\n--' + boundary + '\n'
stopBoundary = '\n--' + boundary + '--\n'
bibFileFields = ['bibFile0', 'bibFile1', 'bibFile2', 'bibFile3']
fileFields = ['inFile'] + bibFileFields
originalBibFileNameList = []

# Import only the absolute minimum here to give maximum response time:
import sys
import os
import tempfile
import time       # needed in delivery and comp phase
import string

# Initialize the temp file name generation:
# Put tmp files into an openly writable dir:
tempfile.tempdir = os.getcwd() + '/tmp'

###########################################################
#
# Definitions of funcs used during INITIAL call via either
# nph-bibconvertcgi.py or nph-bibdocxformcgi.py. These
# are replicated in bibframe.py, but for this bootstrap,
# we need them here:
#
###########################################################

# Is file a mif file? (We assume that the file exists!):
def isMifFile(fileName):
  fd = open(fileName)
  ln = fd.readline()
  fd.close()
  if ln[:8] == '<MIFFile':
    return(1)
  else:
    return(0)

# Is file an rtf file? (We assume that the file exists!):
def isRtfFile(fileName):
  fd = open(fileName)
  ln = fd.readline()
  fd.close()
  if string.lower(ln[:5]) == '{\\rtf':
    return(1)
  else:
    return(0)

def check_user_agent_version(agent):
  isnetscape = 0
  if string.find(agent, "Mozilla") >= 0:
    p = string.split(agent)
    p2 = string.splitfields(p[0], "/")
    version = p2[1]

    isnetscape = (1, version)
  else:
    isnetscape = (0, None)
  return isnetscape

def check_user_agent():
  if os.environ.has_key("HTTP_USER_AGENT"):
    agent = os.environ["HTTP_USER_AGENT"]
    return check_user_agent_version(agent)
  return 0


# Called initially when bib conversion is requested: Sets up the
# screen for subsequent convertion error msgs and results. Note that
# this will call this same routine twice again, because of the load
# instructions for the frames. The argument that gets passed each
# time tell us what to do when we re-enter:

def screenSetupBibConvertion(outFileName):
  sys.stdout.write('Content-type: text/html' + '\n\n')
  sys.stdout.write('<frameset rows="60%,*">\n')
  sys.stdout.write('  <frame name="errors" \
		   src="nph-bibconvertcgi.py?compute+' + outFileName + '">\n')
  sys.stdout.write('  <frame name="document" \
		   src="nph-bibconvertcgi.py?deliver+' + outFileName + '">\n')
  sys.stdout.write('</frameset>\n')
  sys.stdout.write('</html>\n')
  return(outFileName)

# Called initially when bib conversion is requested: Sets up the
# screen for subsequent convertion error msgs and results. Note that
# this will call this same routine twice again, because of the load
# instructions for the frames. The argument that gets passed each
# time tell us what to do when we re-enter:

def screenSetupBibDocxform(outFileName):
  sys.stdout.write('Content-type: text/html' + '\n\n')
  sys.stdout.write('<frameset rows="90%,*">\n')
  sys.stdout.write('  <frame name="errors" \
		   src="nph-bibdocxformcgi.py?compute+' + outFileName + '">\n')
  sys.stdout.write('  <frame name="document" \
		   src="nph-bibdocxformcgi.py?deliver+' + outFileName + '">\n')
  sys.stdout.write('</frameset>\n')
  return(outFileName)
  
# Make the form dictionary, but replace all file fds with their file names:
def createForm(fileFields):
  #***sys.stdout.write('Content-type: multipart/x-mixed-replace;boundary=' + boundary + '\n\n')
  #***sys.stdout.write(startBoundary)
  #***sys.stdout.write('Content-type: text/html\n\n')
  #***sys.stdout.write('<html><body>\n')
  #***sys.stdout.write('Pulling files from you to server...\n')
  #***sys.stdout.flush()

  import bibcgi

  # Get a dict with all the form's fields (this will also init
  # the tempfile directory and file name template):
  form = bibcgi.FileStoringFormContentDict()

  #***sys.stdout.write('done.\n')
  #***sys.stdout.write('</body></html>\n')
  #***sys.stdout.flush()
  #*** Get ready for the actual window-setup at the client:
  #***sys.stdout.write(startBoundary)
  # Close all open file objects for local files and put
  # the file names in the place of the file objects in the form:
  for fileField in fileFields:
    if form.has_key(fileField):
      if fileField in bibFileFields:
        originalBibFileNameList.append(form[fileField][0].sourceFileName)
      form[fileField][0].close()
      form.dict[fileField] = form[fileField][0].name()
  # Prepare the tmp file for the output of the computation later on:
  return(prepOutputFile(form, tempfile.mktemp()))

def prepOutputFile(form, outFileName):
  # Create a lock file so that the doc delivery process invocation can
  # know when the entire return doc is done:
  lockFd = open(outFileName + '-lock', 'w')
  lockFd.write('Not ready!\n')
  lockFd.close()
  fd = open(outFileName, 'w')
  # Write the input form info to the file such that the 2nd invocation
  # (the computation phase) can pick it up there and EVAL it. That's
  # what the backquotes do -- write an evalable version. Same for the
  # List of original file names provided in the form:
  fd.write(`form.dict` + '\n')
  fd.write(`originalBibFileNameList`)
  fd.close()
  return(outFileName)

###########################################################
#
# Definitions of funcs used during the THIRD invocation via either
# nph-bibconvertcgi.py or nph-bibdocxformcgi.py:
#
###########################################################

# The third invocation delivers the document. It uses a lock file
# to ensure it waits for the whole doc (though it shuffles it over
# as it's being produced by the second (computational) invocation:

# Called when this script is re-entered the third time, this time to
# get the final thing and to clean the result file out:

def deliverDocument(fileName):
  try:
    lockFileName = fileName + '-lock'
    # Wait till computation phase has cleaned out the form from the
    # outfile and has started to write output to it. We need to
    # wait till we get at least enough in the file to tell whether
    # it's MIF, RTF or HTML:
    lockFd = open(lockFileName)
    while 1:
      ln = lockFd.readline()
      if ln == 'Done\n':
	break
      if ln == 'Writing\n':
	# Check whether there's enough info to tell what file it is:
	fd = open(fileName)
	contentLn = fd.read(9)
	fd.close()
	if len(contentLn) >= 9:
  	  break
      lockFd.close()
      lockFd = open(lockFileName)
      time.sleep(1)
      continue
    if isMifFile(fileName):
      sys.stdout.write('Content-type: application/x-mif\n\n')
    elif isRtfFile(fileName):
      sys.stdout.write('Content-type: application/rtf\n\n')
    else:
      sys.stdout.write('Content-type: text/html\n\n')
    fd = open(fileName)
    while 1:
      ln = fd.readline()
      if not ln:
	# Check whether the lock file was set to 'Done'. If not, we need to
	# wait for the computation process to produce more document pieces:
        lockFd.close()
	lockFd = open(lockFileName)
	ln = lockFd.readline()
	if ln == 'Done\n':
	  # We delivered the whole thing:
	  break
	else:
	  time.sleep(1)
	  continue
      sys.stdout.write(ln)
  finally:
    os.system('rm ' + fileName + ' ' + lockFileName)

###########################################################
#
# Finally: some actual action
#
###########################################################

# Respond to the request:

sys.stdout.write('HTTP/1.0 200 0K\n')
sys.stdout.write('Server: NCSA/1.4.1\n')

# If caller does not have a Netscape 2.0 browser or better, forget it:

versionTuple = check_user_agent()
if versionTuple:  
  version = versionTuple[1]
  if not versionTuple[0] or string.atoi(version[0]) < 2:
    sys.stdout.write("Sorry, you need Netscape 2.0 or higher, because client/server file transfer is required.<br/>")
    sys.exit()

# If we are being called with the 'compute' or 'deliver' argument, but the
# file with the form data or result are not present, then we are
# being 're-called' as when user uses 'forward' to get back to a result
# page we computed earlier. This is a problem because we don't cache the
# result, so we can't do anything nicer than tell the user to go back
# and resubmit the original form:

if (len(sys.argv) > 1) and\
   ((sys.argv[1] == 'compute') or (sys.argv[1] == 'deliver')) and\
   (not os.path.isfile(sys.argv[2])):
  sys.stdout.write("Please resubmit the original form to refresh the display.<br/>")
  sys.exit()

scriptName = os.path.split(os.environ["SCRIPT_NAME"])[1]

# If we are supposed to convert a bibliography to some target:
if scriptName == 'nph-bibconvertcgi.py':
  # Are we being called with no arg? (i.e. directly from the form submission):
  if len(sys.argv) == 1:
    finalOutFileName = createForm(fileFields)
    screenSetupBibConvertion(finalOutFileName)
    sys.exit()
  # Are we being called to deliver the result?:
  elif sys.argv[1] == 'deliver':
    deliverDocument(sys.argv[2])
    sys.exit()

  # Else computation is now to be done.
# If we are supposed to transform a doc to include a bibl:
elif scriptName == 'nph-bibdocxformcgi.py':
  # Are we being called with no arg? (i.e. directly from the form submission):
  if len(sys.argv) == 1:
    finalOutFileName = createForm(fileFields)
    screenSetupBibDocxform(finalOutFileName)
    sys.exit()
  # Are we being called to deliver the result?:
  elif sys.argv[1] == 'deliver':
    deliverDocument(sys.argv[2])
    sys.exit()
  # Else computation is now to be done.

# If we get here, we are to do the serious part of a computation:
# bibliography or document transformation. We are targeted to the
# error window.
# Give fast response time, even before we import everything:

if os.path.split(os.environ["SCRIPT_NAME"])[1] == 'nph-bibconvertcgi.py':
  feedbackPaneHeader = 'InterBib Bibliography Conversion: Progress, Warnings and Errors'
elif os.path.split(os.environ["SCRIPT_NAME"])[1] == 'nph-bibdocxformcgi.py':
  feedbackPaneHeader = 'InterBib Document Bibliography Processing: Progress, Warnings and Errors'
else:
  feedbackPaneHeader = 'Unknown Operation'

sys.stdout.write('Content-type: text/html\n\n')

sys.stdout.write('<center>\n')
#sys.stdout.write('<img src="../gifs/dllogo-smaller-xparent.gif"/>' +\
#                 feedbackPaneHeader)
sys.stdout.write(feedbackPaneHeader)
sys.stdout.write('</center>\n')

sys.stdout.write('<pre>\n')

sys.stdout.write('Please wait while server loads software...\n')
sys.stdout.flush()

import urllib
import urlparse
import bibdb
import bib2target
import bibframe
import HTMLsupport
html = HTMLsupport
import bibexport
import socket

sys.stdout.write('done.\n')
sys.stdout.flush()

class bibService:

  def __init__(self):
    self.makeIndexes = 1
    self.numOfBibFilesInForm = 4
    self.numOfBibURLsInForm = 4
    self.numOfBibServerFilesInForm = 4
    self.bibFileFldNameBase = 'bibFile'
    self.bibURLFldNameBase = 'bibURL'
    self.bibServerFileNameBase = 'serverBibFile'
    self.bibFileTypeFldNameBase = 'fileType'
    self.bibURLTypeFldNameBase = 'URLType'
    self.fileNameList = []
    self.urlList = []
    self.serverFileList = []
    self.originalBibFileNameList = []
    self.originalServerFileNameList = []
    self.goodBibFileNameList = []
    self.bibliographyDir = os.getcwd() + '/../bibliographies/'
    self.form = None
    self.outFd = sys.stdout
    self.resultFd = None
    self.lockFileName = ''
    self.mifContentType = 'Content-type: application/x-mif\n\n'
    self.htmlContentType = 'Content-type: text/html\n\n'
    self.plainContentType = 'Content-type: text/plain\n\n'
    self.mifFileHeader = "<MIFFile 5.00> # Generated by InterBib\n"
    self.mifBodyStart  = "<Para\n  <PgfTag `Body'>\n > # end of Para\n"
    self.mifTitleStart = "<Para\n  <Pgf\n   <PgfTag `Body'>\n   <PgfAlignment Center>\n   >\n  <ParaLine\n    <String `"
    self.mifTitleEnd = "'>\n  > # end of ParaLine\n> # end of para\n"
    self.mifParaStart   = "<Para\n  <ParaLine \n    <String `"
    self.mifParaEnd  = "'>\n    >\n> # end of para\n"

  # Get a list of the file names that were filled into the form
  # as files delivered from client. We do the necessary file type
  # checking as best we can. What slips through cause parsing errors
  # later on:

  def getBibFileNameList(self):
    res = []
    # Go through all bib file name specs. They are in fields bibFile<n>:
    j = -1
    for i in range(self.numOfBibFilesInForm):
      fileField = self.bibFileFldNameBase + str(i)
      # Was this one filled out?:
      if self.form.has_key(fileField):
	j = j+1
	# Get the corresponding file type spec from the form:
	fileType = self.form[self.bibFileTypeFldNameBase + str(i)][0]
	originalFileName = self.originalBibFileNameList[j]
	originalLocalTmpFileName = self.form[fileField]
	# Make sure the tmp file does not have an extension, so that we
	# can either add a new extension, or can have a name w/o one:
	originalTmpFileDir = os.path.split(originalLocalTmpFileName)[0] + '/'
	originalTmpFileBase = os.path.splitext(os.path.basename(originalLocalTmpFileName))[0]
	originalTmpFileExt = os.path.splitext(os.path.basename(originalLocalTmpFileName))[1]
	if originalTmpFileExt:
 	  localTmpFileName = originalTmpFileBase + originalTmpFileExt[1:]
	else:
	  localTmpFileName = originalLocalTmpFileName
	# Get user's file name w/ extension corrected:
        fileName = self.handleFileTypeSpec(fileType, originalFileName)
	# Make local tmp file have same extension (or none):
	legalExtension = os.path.splitext(os.path.basename(fileName))[1]
	fileName = originalTmpFileDir + localTmpFileName + legalExtension
	os.system('mv ' + originalLocalTmpFileName + ' ' + fileName)
	res.append(fileName)
    return(res)

  # If provided in the form, return file names of bibliographies on this
  # server.  The directory is already added to each name, and it's
  # guaranteed to be an InterBib db.  If this file does not exist, name 
  # ignored:

  def getServerFileNameList(self):
    res = []
    for i in range(self.numOfBibServerFilesInForm):
      serverFileField = self.bibServerFileNameBase + str(i)
      if self.form.has_key(serverFileField):
	fileName = os.path.basename(self.form[serverFileField][0])
	fullFileName = os.path.join(self.bibliographyDir, fileName)
	if os.path.isfile(fullFileName):
	  self.originalServerFileNameList.append(fileName)
	  res.append(fullFileName)
	  self.serverFileList.append(fullFileName)
    return(res)

  # Get a list of the file names resulting from pulling over
  # the files whose URLs were given. We make sure the local
  # files have the correct file extension:

  def getBibURLNameList(self):
    res = []
    # Go through all bib URL specs. They are in fields bibURL<n>:
    for i in range(self.numOfBibURLsInForm):
      urlField = self.bibURLFldNameBase + str(i)
      # Was this one filled out?:
      if self.form.has_key(urlField):
	# Get the corresponding file type spec from the form:
	fileType = self.form[self.bibURLTypeFldNameBase + str(i)][0]
	url = self.form[urlField][0]
	# Now try to get the file behind the URL:
	self.outFd.write("Retrieving '" + url + "' ...\n")
	self.outFd.flush()
 	originalLocalTmpFileName, headerInfo = urllib.urlretrieve(url)
	self.outFd.write('done.\n')
	self.outFd.flush()
	originalTmpFileDir = os.path.split(originalLocalTmpFileName)[0] + '/'
	originalTmpFileBase = os.path.splitext(os.path.basename(originalLocalTmpFileName))[0]
	originalTmpFileExt = os.path.splitext(os.path.basename(originalLocalTmpFileName))[1]
	if originalTmpFileExt:
 	  localTmpFileName = originalTmpFileBase + originalTmpFileExt[1:]
	else:
	  localTmpFileName = originalLocalTmpFileName
        fileNameInURL = urlparse.urlparse(url)[2]
	# Get user's file name w/ extension corrected:
	fileName = self.handleFileTypeSpec(fileType, fileNameInURL)
	# Make local tmp file have same extension (or none):
	legalExtension = os.path.splitext(os.path.basename(fileName))[1]
	fileName = originalTmpFileDir + localTmpFileName + legalExtension
	os.system('mv ' + originalLocalTmpFileName + ' ' + fileName)
	res.append(fileName)
	self.urlList.append(url)
    return(res)

  # Given a legal file name (with or without path), and a fileType
  # ('deriveFromExt', 'strict refer', 'bibtex', etc): check whether the
  # file name extension is consistent with the file type. If not,
  # change the extension, or throw an error if appropriate:

  def handleFileTypeSpec(self, fileType, originalFileName):
    fileName = originalFileName
    # If we are supposed to derive bib file type from its
    # extension, do that:
    if fileType == 'deriveFromExt':
      if not bib2target.checkBibFile(fileName):
	self.outFd.write('Bib format not derivable from file name: ' +
			 fileName + '\n')
	self.outFd.write("   Refer bibliographies need extension '.ref'\n")
	self.outFd.write("   Endnote bibliographies need extension '.enr'\n")
	self.outFd.write("   BibTeX bibliographies need extension '.bib'\n")
	self.outFd.write("   InterBib bibliographies must have no extension.\n")
	self.outFd.write("   OR: use the buttons on the form to specify the format\n")
        self.outFd.flush()
        raise ValueError
    # Else make sure the file extension matches the declared biblio type:
    elif fileType == 'strict refer' and not bib2target.isReferFileName(fileName):
      # Strip any inconsistent extension:
      fileName = os.path.splitext(originalFileName)[0] + '.ref'
    elif fileType == 'endnote' and not bib2target.isEndnoteFileName(fileName):
      # Strip any inconsistent extension:
      fileName = os.path.splitext(originalFileName)[0] + '.enr'
    elif fileType == 'bibtex' and not bib2target.isBibtexFileName(fileName):
      # Strip any inconsistent extension:
      fileName = os.path.splitext(originalFileName)[0] + '.bib'
    elif fileType == 'interbib' and not bib2target.isInterBibFileName(fileName):
      # Strip any inconsistent extension:
      fileName = os.path.splitext(originalFileName)[0]
    return(fileName)

  def run(self):

    # The file name to write results to for subsequent pickup was
    # passed in as an arg. It also contains the form dictionary that
    # was collected during the prep phase. Get it out and truncate
    # the out file:
    resultFileName = sys.argv[2]
    self.resultFd = open(resultFileName)
    self.form = eval(string.strip(self.resultFd.readline()))
    self.originalBibFileNameList = eval(self.resultFd.readline())
    self.resultFd.close()
    # Re-use the file to communicate the final doc to the delivery
    # phase. The 'w' will erase the form info we just extracted:
    self.resultFd = open(resultFileName, 'w')

    self.lockFileName = resultFileName + '-lock'
    # Signal that we have cleared out the out file and are starting to write:
    lockFd = open(self.lockFileName, 'w')
    lockFd.write('Writing\n')
    lockFd.close()

    if self.form.has_key('targetEnv'):
      self.target = string.lower(self.form['targetEnv'][0])
    else:
      target = 'html'

  # Given a bibliography source file (usually in the tmp dir for incoming
  # bib files), return 1 if the bibliography directory contains any file
  # that is equal to the given file. Else return 0. We check file length
  # and equality in that order, to weed out most possibilities quickly:

  def doesBibFileExist(self, sourceFilePath):
    # Check existence by length-equality and content-equality in that order:
    for file in os.listdir(self.bibliographyDir):
      targetFilePath = self.bibliographyDir + file
      if os.stat(sourceFilePath)[6] == os.stat(targetFilePath)[6] and\
         os.system('diff ' + \
		   sourceFilePath + \
		   ' ' + \
		   targetFilePath +\
		   ' >/dev/null 2>/dev/null') == 0:
	return(1)
    return(0)

  # Given a tmp bibl file and a target file name: if the source file
  # is an interbib file, move it to the bibliography pool directory
  # under the specified name. Else create an InterBib db under the
  # target name (w/o the file extension) and also move the source
  # file:

  def retainBibliographyFile(self, sourceFilePath, targetPath):

    targetDbPath = os.path.splitext(targetPath)[0]
    # Put in a lock file while the index is being made, so that the db won't
    # be offered in the search page till the index is complete:
    indexCmd = 'nohup echo "" > ' + targetDbPath + '.lck' +\
	       ' ; ./bibmakeindex ' +\
	       targetDbPath +\
	       ' ; rm ' + targetDbPath + '.lck' +\
	       ' >/dev/null 2>/dev/null'

    if bib2target.isInterBibFileName(sourceFilePath):
      os.system('mv ' +\
		sourceFilePath +\
		' ' +\
		targetDbPath +\
		' ; chmod 644 ' + targetPath +\
		' >/dev/null 2>/dev/null')
      if self.makeIndexes:
	os.system(indexCmd + '&')
    else:
      # Run everything in the background in such a way that this process
      # can exit without disturing the bib import process. This way the
      # Web client sees the process finishing, but things continue at the
      # server site:
      loadCmd = 'nohup ./bibload -q ' +\
		targetDbPath +\
		' ' +\
		sourceFilePath +\
		' ; chmod 644 ' + targetDbPath +\
		' >/dev/null 2>/dev/null'
      cleanUpCmd = 'nohup mv ' +\
		   sourceFilePath +\
		   ' ' +\
		   targetPath +\
		   ' ; chmod 644 ' + targetPath +\
		   ' >/dev/null 2>/dev/null'
      if self.makeIndexes:
	os.system('(' + loadCmd + ' ; ' + cleanUpCmd + ' ; ' + indexCmd + ') &')
      else:
        os.system('(' + loadCmd + ' ; ' + cleanUpCmd + ') &')

  # Given a pathname, try to convert it to a Unix path, if it's DOS format.
  # Return the new path. Note: we don't use the regex module for finding
  # drive info because we don't use it anywhere else, and any module
  # loading will slow the startup time:

  def cnv2Unix(self, pathName):
    # Strip any drive info that might be in the path, i.e. any letter followed by colon
    # at the start of the file name:
    if ((len(pathName) > 1) and
	(pathName[0] in string.letters) and
	(pathName[1] == ':')):
      pathName = pathName[2:]
    pathName = string.joinfields(string.splitfields(pathName, '\\'), '/')
    return pathName

  # Using the lists of original file paths (as specified in the call form)
  # and of URLs pointing to bib files, construct a list of file names
  # that can be used in our bibliography pool directory (i.e. check for
  # duplicate file names, make sure file extensions are correct, etc):

  def findTargetFileNames(self):
    fileNames = []
    index = -1
    for name in self.originalBibFileNameList:
      # Make sure the file name is in Unix format so that all the
      # isolation of basename and extension will work:
      name = self.cnv2Unix(name)
      uniquifier = 1
      index = index + 1
      extension = os.path.splitext(os.path.basename(self.fileNameList[index]))[1]
      newFileName = os.path.splitext(os.path.basename(name))[0]
      while os.path.isfile(self.bibliographyDir + newFileName + extension) or\
	    (newFileName + extension) in fileNames:
	if uniquifier == 1:
  	  newFileName = newFileName + '-' + str(uniquifier)
	else:
	  newFileName = newFileName[:-len(str(uniquifier-1))-1] +\
			'-' +\
			str(uniquifier)
	uniquifier = uniquifier + 1
      fileNames.append(self.bibliographyDir + newFileName + extension)
    for url in self.urlList:
      uniquifier = 1
      index = index + 1
      extension = os.path.splitext(os.path.basename(self.fileNameList[index]))[1]
      path = urlparse.urlparse(url)[2]
      newFileName = os.path.splitext(os.path.basename(path))[0]
      while os.path.isfile(self.bibliographyDir + newFileName + extension) or\
	    (newFileName + extension) in fileNames:
	if uniquifier == 1:
  	  newFileName = newFileName + '-' + str(uniquifier)
	else:
	  newFileName = newFileName[:-len(str(uniquifier-1))-1] +\
			'-' +\
			str(uniquifier)
	uniquifier = uniquifier + 1
      fileNames.append(self.bibliographyDir + newFileName + extension)
    return(fileNames)


  def handleBibFileRetention(self):
    if not self.form.has_key('allowStorage') or\
       self.form['allowStorage'][0] == 'disallow':
      return
    targetPaths = self.findTargetFileNames()
    if not targetPaths:
      return
    for index in range(len(self.goodBibFileNameList)):
      goodFileName = self.goodBibFileNameList[index]
      if goodFileName in self.serverFileList:
	continue
      if not self.doesBibFileExist(goodFileName):
	self.retainBibliographyFile(goodFileName, targetPaths[index])
      else:
	os.system('rm ' + goodFileName)

class convertRequestServer(bibService):

  def __init__(self):
    bibService.__init__(self)
    self.target = 'html'
    self.myBibConverter = None
    self.allErrors = []
    self.allWarnings = []

  def run(self):

    bibService.run(self)
    if self.target == 'mif':
      self.resultFd.write(self.mifFileHeader)
      if self.form.has_key('title') and self.form['title'][0]:
	self.resultFd.write(self.mifTitleStart)
	self.resultFd.write(self.form['title'][0])
	self.resultFd.write(self.mifTitleEnd)
      # Set par format to 'body':
      self.resultFd.write(self.mifBodyStart)

    elif self.target == 'bibtex' or \
	 self.target == 'strict refer' or \
	 self.target == 'endnote':
      if self.form.has_key('title'):
	html.DOCSTART(self.resultFd, title = self.form['title'][0])
	if self.form['title'][0]:
	  self.resultFd.write('<center>')
	  html.H3(self.resultFd, self.form['title'][0])
	  self.resultFd.write('</center>')
        else:
	  html.DOCSTART(self.resultFd, title='InterBib-Converted Bibliography')
      self.resultFd.write('<pre>\n')

    else: # Target is HTML
      if self.form.has_key('title'):
	html.DOCSTART(self.resultFd,
		      title = self.form['title'][0])
	if self.form['title'][0]:
	  self.resultFd.write('<center>')
	  html.H3(self.resultFd, self.form['title'][0])
	  self.resultFd.write('</center>')
        else:
	  html.DOCSTART(self.resultFd, title = 'InterBib-Converted Bibliography')
    try:
      if self.convertBibFiles():
        if self.target == 'html':
          html.BODYEND(self.resultFd)
          html.DOCEND(self.resultFd)
	elif self.target == 'bibtex' or \
	     self.target == 'strict refer' or \
	     self.target == 'endnote':
	  self.resultFd.write('\n</pre>\n')
          html.BODYEND(self.resultFd)
          html.DOCEND(self.resultFd)

	# If appropriate, keep bibliography files:
	#****** Needs debugging. self.handleBibFileRetention()
    finally:
      self.cleanUp()

  def convertBibFiles(self):
    try:

      if self.form.has_key('verboseCheck'):
        self.verbosity = 'verbose'
      else:
        self.verbosity = 'quiet'

      self.fileNameList = self.getBibFileNameList() +\
	  	          self.getBibURLNameList() +\
			  self.getServerFileNameList()

    except (ValueError, socket.error, IOError), info:
	self.outFd.write("Error occurred in getting file or URL: " + str(info) + "\n")
	self.outFd.write("*** Quitting ***\n")
	return(0)

    if self.fileNameList:
      # Get a converter to do the work. Set it up so that it
      # writes errors to stdout (i.e. to the client's error window
      # which is the recipient of our outpourings right now):
      self.myBibConverter = bib2target.bibConverter(targetEnv = self.target,
						    outFd = self.resultFd,
						    errorFd = sys.stdout,
						    verbosity = self.verbosity)
      self.myBibConverter.setbibFiles(self.fileNameList)

      self.outFd.write('Importing bibliography material...\n')
      self.outFd.flush()
      if self.form.has_key('dupKey'):
	if self.form['dupKey'][0] == 'autoResolve':
	  dupBehavior = bibdb.autoResolveRecordConflicts
	elif self.form['dupKey'][0] == 'overwrite':
	  dupBehavior = bibdb.overwriteDuplicates
	elif self.form['dupKey'][0] == 'discard':
	  dupBehavior = bibdb.discardDuplicates
	elif self.form['dupKey'][0] == 'error':
	  dupBehavior = bibdb.duplicatesCauseErrors
      else:
	dupBehavior = bibdb.autoResolveRecordConflicts
      if self.form.has_key('annotationInstructions') and\
	 self.form['annotationInstructions'][0] == 'includeAnnotations':
	annote = 1
      else:
	annote = 0
      try:
        self.myBibConverter.bibliographyImport(duplicatePolicy = dupBehavior,
	  		               dbIdentifiers = self.originalBibFileNameList +\
		                       self.urlList +\
		                       self.originalServerFileNameList)
					       
	self.allErrors = self.allErrors + self.myBibConverter.errors
	self.allWarnings = self.allWarnings + self.myBibConverter.warnings

        #**** Maybe change this to only refuse to save if more than 1% are bad:
	if not self.myBibConverter.errors:
	  self.goodBibFileNameList = self.fileNameList
      except bibdb.DuplicateRecordKey, info:
	self.outFd.write("Error: key '" + info + "' occurs in multiple databases.\n\
	 Check whether a more forgiving setting of the duplicate \n\
	 key behavior button in the form fills your needs.\n\
	 *** Quitting ***")
        return(0)
      self.outFd.write('Done importing bibliography material.\n')
      self.outFd.flush()
    
      needOutcomeSummary = False
      try:
	if self.target == 'bibtex':
	  exporter = bibexport.BibtexExporter( db=self.myBibConverter.db,
					       verbosity = self.verbosity)
	  exporter.exportBibliography( self.resultFd, sys.stdout )
	  needOutcomeSummary = True
	elif self.target == 'strict refer':
	  exporter = bibexport.ReferExporter( db=self.myBibConverter.db,
					      verbosity = self.verbosity)
	  exporter.exportBibliography( self.resultFd, sys.stdout )
	  needOutcomeSummary = True
	elif self.target == 'endnote':
	  exporter = bibexport.EndnoteExporter( db=self.myBibConverter.db,
						verbosity = self.verbosity)
	  exporter.exportBibliography( self.resultFd, sys.stdout )
	  needOutcomeSummary = True
	else:
	  # Do the conversion, but don't have the translation insert
	  # HTML headers and such (the resultStandalone part):
	  self.myBibConverter.bib2target(resultStandalone = 0, annote=annote)
	return(1)

      finally:

	# Translations to HTML do their own 
	# error and warning summary in bib2target.
	# But for xlation to Refer or Bibtex we need
	# output how it went:

	if needOutcomeSummary:

	  # Collect the errors/warnings from the 
	  # export process:

	  self.allErrors = self.allErrors + exporter.errors
	  self.allWarnings = self.allWarnings + exporter.warnings

	  # If we operated 'verbose' then errors
	  # and warnings were output en route. In
	  # 'silent' mode we are, well, silent. But
	  # for 'quiet' mode we need to show the
	  # errors:

      	  if (self.verbosity == 'quiet'):

 	    if len(self.allErrors) > 0:
 	      sys.stdout.write("\n")
 	      for error in self.allErrors:
 		sys.stdout.write(error + "\n")
    
	  if (self.verbosity != 'silent'):

 	    # Error and warning summary:
      	    
 	    if self.allErrors:
 	    	sys.stdout.write("\n\nTotal Errors: " + \
				 str(len(self.allErrors)) + "\n")
 	    else:
 	    	sys.stdout.write("\n\nTotal Errors: none\n")

 	    if self.allWarnings:
 	    	sys.stdout.write("Total Warnings: " + \
				 str(len(self.allWarnings)) + "\n")
 	    else:
 	    	sys.stdout.write("Total Warnings: none\n")

	    sys.stdout.write("Total entries: " + \
			     str(self.myBibConverter.getRecsImported()) + "\n")

          sys.stdout.flush()

    else:
      print 'Error: No bibliography files submitted, or they were not found.\n'
      return(0)
  
  def cleanUp(self):
    self.resultFd.flush()
    # Release the lock file on the output document so that the delivery
    # process knows that we are done:
    lockFd = open(self.lockFileName, 'w')
    lockFd.write('Done\n')
    lockFd.close()
    for file in self.fileNameList:
      # If we are retaining bibliographies and this one was good, don't
      # delete it, because it will be taken care of by the backgroun
      # process doing the transfer:
      if self.form.has_key('allowStorage') and\
	 self.form['allowStorage'][0] == 'allow' and\
	 file in self.goodBibFileNameList:
	continue
      if os.path.isfile(file) and not file in self.serverFileList:
        os.system('rm ' + file)

class docxformRequestServer(bibService):

  def __init__(self):
    bibService.__init__(self)
    self.bibMaker = None
    self.fileNameList = []

  def transformDocument(self):
    try:
      self.fileNameList = self.getBibFileNameList() +\
	 	          self.getBibURLNameList() +\
			  self.getServerFileNameList()

      if self.form.has_key('verboseCheck'):
        self.verbosity = 'verbose'
      else:
        self.verbosity = 'quiet'

    except (ValueError, socket.error, IOError), info:
      self.outFd.write("Error occurred in getting file or URL: " + str(info) + "\n")
      self.outFd.write("*** Quitting ***\n")
      return(0)
    if not self.errorCheck():
      raise ValueError
    # Get a machine to do the work:
    self.bibMaker = bibframe.bibMaker(targetEnv = self.target,
    				      outFd = self.resultFd,
    				      errorFd = sys.stdout,
                                      runQuiet = (self.verbosity != 'verbose'))
    					
    self.bibMaker.setbibFiles(self.fileNameList)
    self.bibMaker.inFile = self.form['inFile']    
    self.bibMaker.insertionMarker = self.form['bibAnchor'][0]
    if not self.bibMaker.fixInFileFormats():
      return(0)
    self.outFd.flush()
    if self.form.has_key('dupKey'):
      if self.form['dupKey'][0] == 'autoResolve':
	dupBehavior = bibdb.autoResolveRecordConflicts
      elif self.form['dupKey'][0] == 'overwrite':
	dupBehavior = bibdb.overwriteDuplicates
      elif self.form['dupKey'][0] == 'discard':
	dupBehavior = bibdb.discardDuplicates
      elif self.form['dupKey'][0] == 'error':
	dupBehavior = bibdb.duplicatesCauseErrors
    else:
      dupBehavior = bibdb.autoResolveRecordConflicts
    self.outFd.write('Importing bibliography material...\n')
    if not self.bibMaker.bibliographyImport(duplicatePolicy = dupBehavior,
				            dbIdentifiers = self.originalBibFileNameList +\
				                            self.urlList +\
					                    self.originalServerFileNameList):

      self.outFd.write('***Quitting***.\n')
      return(0)
    if not self.bibMaker.errors:
      self.goodBibFileNameList = self.fileNameList
    self.outFd.write('Done importing bibliography material.\n')
    self.outFd.flush()
    self.bibMaker.refFormat = 'GenericStyle'
    if self.form.has_key('style'):
      if self.form['style'][0] == 'IEEE':
    	self.bibMaker.citationFormat = 'NumericCiteAppearanceOrder'
      elif self.form['style'][0] == 'ACM':
        self.bibMaker.citationFormat = 'NumericCiteAuthorAlphaOrder'
      else:
	self.bibMaker.citationFormat = self.form['style'][0]
    else:
      self.bibMaker.citationFormat = 'NumericCiteAppearanceOrder'
    if self.form.has_key('citationOpener') and\
       self.form.has_key('citationCloser'):
      self.bibMaker.setDocumentCitationDelimiters(self.form['citationOpener'][0],
						  self.form['citationCloser'][0])
    else:
      self.bibMaker.setDocumentCitationDelimiters('[', ']')
    if self.form.has_key('annotationInstructions') and\
       self.form['annotationInstructions'][0] == 'includeAnnotations':
      annote = 1
    else:
      annote = 0
    # The file header will be emitted just before the actual, final
    # result document gets sent:
    self.bibMaker.processFile(annote=annote)
  
  def run(self):
   
    bibService.run(self)
    try:
      self.transformDocument()
      if self.target == 'html':
        html.BODYEND(self.resultFd)
        html.DOCEND(self.resultFd)
      # If appropriate, keep the bibliography files:
      self.handleBibFileRetention()
    finally:
      self.cleanUp()

  def errorCheck(self):
    allOK = 1
    if not self.form.has_key('bibAnchor'):
      self.outFd.write('ERROR: Bibliography anchor field must be filled in!\n')
      self.outFd.write('This is a word in your document. The system will look\n')
      self.outFd.write('for the last occurrence of that word and stick\n')
      self.outFd.write('your bibliography there.\n')
      allOK = 0
    if not self.form.has_key('inFile'):
      self.outFd.write('ERROR: Must provide a file to process!\n')
      allOK = 0
    if not self.fileNameList:
      self.outFd.write('ERROR: No bibliographies provided, or they were not found! Fill in the field(s) correctly.\n')
      allOK = 0
    if not allOK:
      self.outFd.write('</pre></body></html>')
      self.cleanUp()
      return(0)
    else:
      return(1)

  def cleanUp(self):
    self.resultFd.flush()
    # Release the lock file on the output document so that the delivery
    # process knows that we are done:
    lockFd = open(self.lockFileName, 'w')
    lockFd.write('Done\n')
    lockFd.close()
    for file in self.fileNameList:
      # If we are retaining bibliographies and this one was good, don't
      # delete it, because it will be taken care of by the backgroun
      # process doing the transfer:
      if self.form.has_key('allowStorage') and\
	 self.form['allowStorage'][0] == 'allow' and\
	 file in self.goodBibFileNameList:
	continue
      if os.path.isfile(file) and not file in self.serverFileList:
        os.system('rm ' + file)
    if self.form.has_key('inFile') and os.path.isfile(self.form['inFile']):
      os.system('rm ' + self.form['inFile'])
    if self.bibMaker:
      self.bibMaker.cleanUp()

if os.path.split(os.environ["SCRIPT_NAME"])[1] == 'nph-bibconvertcgi.py':
  server = convertRequestServer()
elif os.path.split(os.environ["SCRIPT_NAME"])[1] == 'nph-bibdocxformcgi.py':
  server = docxformRequestServer()
else:
  print "CGI service '" +\
	os.path.split(os.environ["SCRIPT_NAME"])[1] +\
	"' not implemented."
  html.BODYEND(sys.stdout)
  html.DOCEND(sys.stdout)
  sys.exit()

server.run()
